﻿<#
.Synopsis
 Add/Remove registry entries to lock Windows 10 Version.

.Description
 Add/Remove registry entries to lock Windows 10 Version.
 Works on versions 1809+ Pro+.
  

.Parameter Remove
 Will remove the two entries re-enabling WindowsUpdate to install
 feature updates to the next version(s).

.Outputs
 Two Registry Entries 

.Notes
   Programmer   : RetiredGeek (@askWoody.com) & 
                              (@stackoverflow.com) 
             aka: ComputerMentor
   Created      : 26 Jun 2020
   Last Updated : 
   Current Vers : 1.0
   Tested On    : Windows Powershell 5.1.18362 
                  PowerShell Core 7.0.2

.Example
 [d:\path\]Set-LockWinVersion.ps1

 Will create the necessary registry entries to lock the Windows
 Version to the current release.

.Example
 [d:\path\]Set-LockWinVersion.ps1 -ReleaseNo 1903

 Will create the necessary registry entries to lock the Windows
 Version to the specified (1903) release. Thus if you are on
 release 1803 it will allow updates to 1809 then 1903.

.Example
 [d:\path\]Set-LockWinVersion.ps1 -Remove

 Will remove the version locking entries from the registry.

.Link
 https://www.computerworld.com/article/3564158/microsoft-nixes-update-deferral-settings-but-gives-us-a-targetreleaseversioninfo.html
#>


# Two blank linea above required for Comment-Based Help

#Requires -RunAsAdministrator

[CmdletBinding()]

Param (
     [Parameter(Mandatory=$False)]
       [Switch] $Remove,
     [Parameter(Mandatory=$False)]
       [ValidateSet("1809","1903","1909","2004")]
       [String] $ReleaseNo
)

Function Remove-RegistryItem {

[CmdletBinding()]
Param (
     [Parameter(Mandatory=$True)]
       [String] $RegPath,
     [Parameter(Mandatory=$False)]
       [String] $KeyName = ""
)

   If ((Test-Path -Path $RegPath)) {    
    
     If ($KeyName -ne '') { #Delete the key Value
       $RIArgs = @{Path         = $RegPath 
                   Name         = $KeyName
                   Force        = $True
                   ErrorAction  = 'Stop'}
       Try   {Remove-ItemProperty @RIArgs | Out-Null}
       Catch {$Message = "$RegPath\$KeyName not found!"
              $Null = & $TermMsg}
     }
     Else {     #Remove the Registry Path & Children!
       $RIArgs = @{Path         = $RegPath 
                   Force        = $True
                   Recurse      = $True
                   ErrorAction = 'Stop'}
       Try   {Remove-Item @RIArgs}
       Catch {$Message = "$RegPath"
              $Null = & $TermMsg}
     }

   } #End If ((Test-Path...

} #End Function Remove-RegistryItem

Function Update-Registry {

[CmdletBinding()]
Param (
     [Parameter(Mandatory=$True)]
       [String] $RegPath,
     [Parameter(Mandatory=$False)]
       [String] $KeyName = "",
     [Parameter(Mandatory=$False)]
       [String] $KeyValue,
     [Parameter(Mandatory=$False)]
       [String] $KeyType
)

   If (-not(Test-Path -Path $RegPath)) {
     Write-Verbose -Message  "$RegPath does NOT Exist!"
     Write-Verbose -Message "Creating $RegPath"
     New-Item -Path $RegPath -Force | Out-Null
   }
   
   #Set the key Value
   If ($KeyName -ne '') {
     $NIPArgs = @{Path         = $RegPath 
                  Name         = $KeyName
                  Value        = $KeyValue
                  PropertyType = $KeyType
                  Force        = $True}
     New-ItemProperty @NIPArgs | Out-Null
   }

} #END Function Update-Registry

#-----------------------  Main Program  -----------------------

Add-Type -AssemblyName "System.Windows.Forms"

$TermMsg = {
  [Windows.Forms.MessageBox]::Show($Message,
                                   "Program Terminated:", 
  [Windows.Forms.MessageBoxButtons]::OK , 
  [Windows.Forms.MessageBoxIcon]::Information) | Out-Null}

If ($Remove.IsPresent) {

  $RRArgs = @{RegPath = "Registry::HKLM\SOFTWARE\Policies\" +
                         "Microsoft\Windows\WindowsUpdate"
              KeyName = "TargetReleaseVersion"}
  Remove-RegistryItem @RRArgs

  $RRArgs.KeyName = "TargetReleaseVersionInfo"
  Remove-RegistryItem @RRArgs

}  #End If

Else {

  $GIPArgs = @{Path = "Registry::HKLM\SOFTWARE\Microsoft\" +
                      "Windows NT\CurrentVersion"
              }

  $WinVer = Get-Item @GIPArgs

  $ProdName  = $WinVer.getvalue('ProductName')
  $ReleaseID = $Winver.getvalue('ReleaseId')

  If ($ProdName -like "*home*") {

    $Message = "You are running Windows 10 Home" +
               "this only works on Pro+!"
    $Null = & $TermMsg

    Exit
  }

  If ($ReleaseNo -eq "") { $ReleaseNo = $ReleaseID }

  If ($ReleaseNo -lt $ReleaseID) {

    $Message = "The requested ReleaseNo ($ReleaseNo)`n" +
               "is less than the Current Release ($ReleaseID)!"
    $Null = & $TermMsg

    Exit
  }

  $URArgs = @{RegPath  = "Registry::HKLM\SOFTWARE\Policies\" +
                         "Microsoft\Windows\WindowsUpdate"
              KeyName  = "TargetReleaseVersion"
              KeyValue = "1"
              KeyType  = "DWord"}
  
  Update-Registry @URArgs
  
  <#+------------------------------------------------------------+
    | Valid values for below: 1809,1903,1908,2004                |
    +------------------------------------------------------------+
  #>
  
  $URArgs = @{RegPath  = "Registry::HKLM\SOFTWARE\Policies\" +
                         "Microsoft\Windows\WindowsUpdate"
              KeyName  = "TargetReleaseVersionInfo"
              KeyValue = "$ReleaseNo"
              KeyType  = "String"}
  
  Update-Registry @URArgs
  
} #End Else